#!/usr/bin/env python2.7
# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This script rebuilds all the profiles in the repository.

This script parses a file of GUIDs, one per line with the pdb filename. e.g.:

4033A4DE6936470BAB02F14DCE270B772 ntkrnlmp.pdb
AED9ED72BEE246CAAC9A587B970A8E0C1 ntkrnlpa.pdb
C77DDDA381D246EDBE11A332456F9FBE1 ntkrpamp.pdb
...

We then check if we have the pdb file in the repository's src/pdb/ directory. If
not we download it.

We then parse the pdb file and store the result in the repository under:

ntoskrnl.exe/GUID/

If that file does not exist.
"""

__author__ = "Michael Cohen <scudette@google.com>"

import argparse
import json
import gzip
import pdb
import os
import time
import traceback
import multiprocessing

from rekall import config
from rekall import interactive
from rekall import plugin
from rekall import utils

session = interactive.ImportEnvironment(verbose="debug")

PARSER = argparse.ArgumentParser(
    description='Rebuild the profile repository.')

PARSER.add_argument('path_to_guids',
                    help='Path to the GUIDs file.')

PARSER.add_argument('--rebuild', default=False, action='store_true',
                    help='Rebuild all profiles.')

PARSER.add_argument('--index', default=None,
                    help='Rebuild all indexes (module name or "all").')

PARSER.add_argument('--inventory', default=False, action='store_true',
                    help='Rebuild repository inventory.')

PARSER.add_argument('--generate_help', default=False, action='store_true',
                    help='Regenerate the help profile')

PARSER.add_argument('--debug', default=False, action='store_true',
                    help='Break on exception')


NUMBER_OF_CORES = multiprocessing.cpu_count()

PDB_TO_SYS = {
    "conhost.pdb": "conhost",
    "dnsrslvr.pdb": "dnsrslvr",
    "lsasrv.pdb": "lsasrv",
    "ntdll.pdb": "ntdll",
    "ntkrnlmp.pdb": "nt",
    "ntkrnlpa.pdb": "nt",
    "ntkrpamp.pdb": "nt",
    "ntoskrnl.pdb": "nt",
    "tcpip.pdb": "tcpip",
    "tcpip6.pdb": "tcpip",
    "tcpipreg.pdb": "tcpip",
    "wdigest.pdb": "wdigest",
    "win32k.pdb": "win32k",
    }

FILENAMES_TO_TRY = [
    "ntkrnlmp.pdb",
    "ntkrnlpa.pdb",
    "ntkrpamp.pdb",
    "ntoskrnl.pdb",
    "tcpip6.pdb",
    "tcpip.pdb",
    "tcpipreg.pdb",
    "win32k.pdb",
    "ntdll.pdb",
    ]


def EnsurePathExists(path):
    try:
        os.makedirs(path)
    except OSError:
        pass


def BuildProfile(pdb_filename, profile_path, metadata):
    print("Parsing %s into %s" % (pdb_filename, profile_path))
    try:
        session.RunPlugin(
            "parse_pdb",
            pdb_filename=pdb_filename,
            output=profile_path,
            metadata=metadata)

        # Gzip the output
        with gzip.GzipFile(filename=profile_path+".gz", mode="wb") as outfd:
            outfd.write(open(profile_path).read())
    except Exception:
        print("Error during profile %s" % pdb_filename)
        print("You can run it manually: "
              "rekall parse_pdb --pdb_filename=%r --output=%r --metadata=%r" %
              (pdb_filename, profile_path, metadata))
        traceback.print_exc()

    finally:
        os.unlink(profile_path)


def BuildAllProfiles(guidfile_path, rebuild=False, reindex=None):
    changed_files = set()
    new_filenames = {}
    unsuccessful = set()

    pool = multiprocessing.Pool(NUMBER_OF_CORES)
    for line in open(guidfile_path):

        line = line.strip()
        if line.startswith("#") or line == "":
            continue

        try:
            guid, pdb_filename = line.split(" ", 2)

            # We dont care about this pdb.
            if pdb_filename not in PDB_TO_SYS:
                continue

        except ValueError:
            guid, pdb_filename = line, None

        # Fetch the pdb from the MS symbol server.
        pdb_path = os.path.join("src", "pdb")
        pdb_out_filename = os.path.join(pdb_path, "%s.pdb" % guid)

        if not pdb_filename:
            for candidate_filename in FILENAMES_TO_TRY:
                try:
                    session.RunPlugin(
                        "fetch_pdb",
                        pdb_filename=candidate_filename, guid=guid,
                        dump_dir=pdb_path)
                    os.rename(os.path.join(pdb_path, candidate_filename),
                              pdb_out_filename)
                except Exception:
                    unsuccessful.add(guid)
                    continue

                pdb_filename = candidate_filename
                new_filenames[guid] = pdb_filename
                break

        if not pdb_filename:
            unsuccessful.add(guid)
            continue

        profile_path = os.path.join(PDB_TO_SYS[pdb_filename], "GUID", guid)
        # Do not export the profile if we already have it.
        if rebuild or not os.access(profile_path + ".gz", os.R_OK):
            # Dont bother downloading the pdb file if we already have it.
            if not os.access(pdb_out_filename, os.R_OK):
                session.RunPlugin(
                    "fetch_pdb",
                    pdb_filename=pdb_filename, guid=guid,
                    dump_dir=pdb_path)

                try:
                    os.rename(os.path.join(pdb_path, pdb_filename),
                              pdb_out_filename)
                except OSError:
                    unsuccessful.add(guid)
                    continue

            implementation = os.path.splitext(
                PDB_TO_SYS[pdb_filename])[0].capitalize()

            metadata = dict(
                ProfileClass=implementation,
                PDBFile=pdb_filename,
                )

            changed_files.add(PDB_TO_SYS[pdb_filename])
            pool.apply_async(
                BuildProfile,
                (pdb_out_filename, profile_path, metadata))

        if reindex == PDB_TO_SYS[pdb_filename] or reindex == "all":
            changed_files.add(PDB_TO_SYS[pdb_filename])

    # Wait here until all the pool workers are done.
    pool.close()
    pool.join()

    if new_filenames:
        print("Found %d new file names:" % len(new_filenames))
        for guid, filename in sorted(new_filenames.items()):
            print("%s %s" % (guid, filename))

    if unsuccessful:
        print("Unable to download pdbs for:")
        for guid in sorted(unsuccessful):
            print(guid)

    return changed_files


def RebuildHelp():
    """Rebuilds the plugin help profile."""
    help_dict = {}
    plugin_metadata = {}
    result = {
        "$METADATA": dict(
            Type="Profile",
            ProfileClass="PluginHelp"
            ),
        "$HELP": help_dict,
        "$PLUGINS": plugin_metadata,
        }

    for cls in plugin.Command.classes.values():
        session.report_progress("Rebuilding profile help: %s.", cls.__name__)

        # Use the info class to build docstrings for all plugins.
        info_plugin = session.plugins.info(cls)

        default_args = [
            x for x, _ in info_plugin.get_default_args()]

        doc = utils.SmartUnicode(info_plugin)
        help_dict[cls.__name__] = [default_args, doc]

        command_metadata = config.CommandMetadata(cls)
        plugin_metadata[cls.__name__] = command_metadata.Metadata()

    with gzip.GzipFile(filename="help_doc.gz", mode="wb") as outfd:
        outfd.write(json.dumps(result))


def RebuildInventory():
    old_inventory = {}
    try:
        with gzip.GzipFile(filename="inventory.gz", mode="rb") as outfd:
            old_inventory = json.load(outfd)["$INVENTORY"]
    except IOError:
        pass

    inventory = {}
    metadata = dict(Type="Inventory",
                    ProfileClass="Inventory")

    result = {
        "$METADATA": metadata,
        "$INVENTORY": inventory,
    }

    modified = False
    for root, _, files in os.walk('./'):
        for filename in files:
            if filename.endswith(".gz"):
                path = os.path.join(root, filename)
                profile_name = os.path.join(root[2:], filename[:-3])

                file_modified_time = os.stat(path).st_mtime
                try:
                    last_modified = old_inventory[profile_name]["LastModified"]

                    # If the current file is not fresher than the old file, we
                    # just copy the metadata from the old profile. Allow 1
                    # second grace for float round ups.
                    if file_modified_time <= last_modified + 1:
                        inventory[profile_name] = old_inventory[profile_name]
                        continue

                except KeyError:
                    pass

                session.report_progress("Adding %s to inventory", path)
                with gzip.GzipFile(filename=path, mode="rb") as fd:
                    data = json.load(fd)

                    inventory[profile_name] = data["$METADATA"]
                    inventory[profile_name][
                        "LastModified"] = file_modified_time
                    modified = True

    if modified:
        # Update the last modified time for the inventory itself.
        result["LastModified"] = time.time()

    with gzip.GzipFile(filename="inventory.gz", mode="wb") as outfd:
        outfd.write(utils.PPrint(result))


def main():
    # Get a renderer for our own output.
    renderer = session.GetRenderer()
    renderer.start()

    if FLAGS.inventory:
        RebuildInventory()
        return

    changes = BuildAllProfiles(FLAGS.path_to_guids, rebuild=FLAGS.rebuild,
                               reindex=FLAGS.index)

    # If the files have changed, rebuild the indexes.
    for change in changes:
        print("Rebuilding profile index for %s" % change)
        output_filename = os.path.join(change, "index")
        session.RunPlugin(
            "build_index",
            output=output_filename,
            spec=os.path.join(change, "index.yaml"))

        # Gzip the output
        with gzip.GzipFile(filename=output_filename+".gz", mode="wb") as out:
            with open(output_filename, 'rb') as fd:
                out.write(fd.read())

        os.unlink(output_filename)

    if FLAGS.generate_help:
        RebuildHelp()

    RebuildInventory()


FLAGS = PARSER.parse_args()

if __name__ == "__main__":
    try:
        main()
    except Exception:
        if FLAGS.debug:
            pdb.post_mortem()



#!/usr/bin/env python

# Rekall
# Copyright 2014 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@google.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
import argparse
import collections
import json
import os
import subprocess
import sys
import time
import yaml
import smtplib
from email.mime import text

PARSER = argparse.ArgumentParser()
PARSER.add_argument("-c", "--config", required=True,
                    help="The config file to parse.")


def RunTests(configuation, timestamp):
    result = 0
    test_prog = configuation["TAP"]["testsuite"]
    configs = configuation["TAP"]["test_directories"]
    control = configuation["control"]
    for path in configs:
        args = [test_prog, "-c", os.path.join(path, "tests.config"),
                "--output_dir", timestamp, "--control", control]
        print "Running test %s" % " ".join(args)
        res = subprocess.call(args)
        print "Test returned status ", res

        if res != 0:
            result = res

    return result


def RenderResult(configs, timestamp):
    results = collections.OrderedDict()
    all_tests = set()
    for config in configs:
        name = os.path.basename(config)
        out_path = os.path.join(config, timestamp)

        results[name] = {}
        result = json.load(open(os.path.join(out_path, "results")))
        for success in result["passes"]:
            all_tests.add(success)
            results[name][success] = True

        for fail in result["fails"]:
            all_tests.add(fail)
            results[name][fail] = False


    out = """
<html>
<head>
<!-- Latest compiled and minified CSS -->
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css">

<!-- Optional theme -->
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap-theme.min.css">

<!-- Latest compiled and minified JavaScript -->
<script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/js/bootstrap.min.js"></script>

</head>
<body><h1>TAP run @ %s</h1>
<table class="table">
   <tr><th>Test</th>
""" % (timestamp)

    for name in results:
        out += "<th>%s</th>" % name

    out += "</tr>"

    for test in sorted(all_tests):
        out += "<tr><td>%s</td>" % test
        for name in results:
            test_result = results[name].get(test)
            if test_result is None:
                out += "<td></td>"
            else:
                css_class = ("btn btn-success btn-sm"
                             if test_result else "btn btn-danger btn-sm")
                text_value = "PASS" if test_result else "FAIL"
                suffix = "" if test_result else ".diff"

                # Add an entry for each test in this suite.
                out += """
<td>
  <a href='../%s/%s/%s%s'>
    <button class='%s'>%s</button>
  </a>
</td>""" % (name, timestamp, test, suffix, css_class, text_value)

        out += "</tr>\n"
    out += "</table></body></html>"

    return out


def SendEmail(configuation, passed=True):
    """Send an email with the status of the test run."""
    with open(configuation["control"]) as fd:
        data = json.load(fd)
        data.update(configuation["ProjectInfo"])
        data.update(configuation["Email"])
        duration = time.time() - configuation["start_time"]
        data["duration"] = "%d minutes and %d seconds" % divmod(duration, 60)
        data["short_hash"] = data["head_commit"]["id"][:7]

    if passed:
        data["icon"] = "success.png"
        data["color_style"] = "background-color:#baecb7;color:#32a32d;"
        data["message"] = "Build %s passed" % data["short_hash"]
    else:
        data["icon"] = "failed.png"
        data["color_style"] = "background-color:#fdcdce;color:#df192a;"
        data["message"] = "Build failed"

    TEMPLATE = """
    <div>
      <table style="padding:0px;border:0px;width:100%;color:#606060;font-size:20px;margin-bottom:15px;margin-top:15px">
        <tbody>
          <tr style="padding:0px;border:0px">
            <td style="padding:0px;border:0px;vertical-align:middle">
              <img src="{logo_url}" height="25">
              <span style="vertical-align:middle;margin-left:3px">
                <strong><a href="{project_url}" style="text-decoration:underline;color:#606060" target="_blank">{project_name}</a></strong>
              </span>
            </td>
          </tr>
        </tbody>
      </table>
      <div style="border-radius:5px;padding:0px;width:570px;font-size:13px">
        <div>
         <table style="padding:0px;border:0px;width:100%;border-spacing:0">
           <thead>
             <tr style="padding:0px;border:0px;font-weight:700;font-size:18px;{color_style}">
               <td style="border:0px;border-top:1px solid #808080;border-bottom:1px solid #adadad;width:50px;padding:0px;text-align:center;vertical-align:middle;padding-top:5px;border-left:1px solid #606060;border-top-left-radius:5px">
                 <div style="width:25px;min-height:30px;margin-left:15px;margin-top:0px;vertical-align:middle">
                   <img height="25" src="{root_url}/images/{icon}" width="25">
                 </div>
               </td>
               <td style="border:0px;padding:0px 20px 0px 0px;vertical-align:middle;border-top:1px solid #808080;border-bottom:1px solid #adadad">
                 <span style="display:inline-block;margin-top:12px;vertical-align:middle">
                   <a href="{dashboard_url}" style="font-weight:bold;text-decoration:underline;{color_style}" target="_blank">
                     {message}
                   </a>
                 </span>
               </td>
               <td align="right" style="border:0px;font-weight:normal;font-size:12px;padding:0px 20px 0px 0px;vertical-align:middle;border-top:1px solid #808080;border-bottom:1px solid #adadad;border-right:1px solid #606060;border-top-right-radius:5px">
                  <div style="vertical-align:middle;padding:0px;display:inline-block;width:20px;min-height:20px">
                     <img height="20" src="{root_url}/images/stopwatch-silhouette-hi.png" width="20">
                  </div>
                 <span style="vertical-align:middle">{duration}</span></td>
             </tr>
          </thead>
          <tbody style="margin-bottom:40px">
          <tr style="padding:0px;border:0px">
              <td style="border:0px;height:20px;width:50px;padding:0px;border-left:1px solid #adadad;padding-top:20px;padding-bottom:5px;text-align:center"></td>
              <td style="border:0px;color:#808080;padding:10px 20px 10px 0px;height:20px;padding-top:20px;padding-bottom:5px;">
               <strong>{head_commit[author][name]}</strong>
              </td>
              <td style="border:0px;color:#808080;padding:10px 20px 10px 0px;height:20px;padding-top:20px;padding-bottom:5px; border-right:1px solid #adadad;">
                <a href="{head_commit[url]}">Commit {short_hash}</a>
              </td>
          </tr>
          <tr style="padding:0px;border:0px">
            <td style="border:0px;height:20px;width:50px;padding:0px;border-left:1px solid #adadad;border-bottom-left-radius:5px;border-bottom:1px solid #adadad">&nbsp;</td>
            <td colspan="2" style="border:0px;color:#808080;padding:10px 20px 10px 0px;height:20px;border-right:1px solid #adadad;padding-bottom:20px;padding-top:0px;border-bottom:1px solid #adadad;border-bottom-right-radius:5px">
              {head_commit[message]}
            </td>
       </tr>
      </tbody>
    </table>
    </div>
    </div>
    """
    message = text.MIMEText(TEMPLATE.format(**data), "html")
    message["Subject"] = data["message"]
    message["From"] = "noreply@tap.rekall-forensic.com"
    message["To"] = data["destination_email"]

    session = smtplib.SMTP(configuation["Email"]["smtp_server"],
                           configuation["Email"]["smtp_server_port"])
    session.ehlo()
    session.starttls()
    session.login(configuation["Authentication"]["gmail_account"],
                  configuation["Authentication"]["gmail_smtp_password"])
    session.sendmail(message["From"], [message["To"]], message.as_string())
    session.quit()


def main():
    args = PARSER.parse_args()

    timestamp = time.strftime("%Y%m%d%H%M")
    configuation = yaml.safe_load(open(args.config))
    configuation["start_time"] = time.time()

    result = RunTests(configuation, timestamp)

    html_page = RenderResult(configuation["TAP"]["test_directories"], timestamp)
    output_path = configuation["TAP"]["output_directory"]

    with open(os.path.join(output_path, timestamp + ".html"), "wb") as fd:
        fd.write(html_page)

    with open(os.path.join(output_path, "latest.html"), "wb") as fd:
        fd.write(html_page)

    # Now send the email
    SendEmail(configuation, result == 0)

    sys.exit(result)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Rekall
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
"""Meta-script for pulling in all Rekall components."""

__author__ = "Michael Cohen <scudette@gmail.com>"

import versioneer

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

rekall_description = "Rekall Memory Forensic Framework"

MY_VERSION = versioneer.get_version()

setup(
    name="rekall",
    version=MY_VERSION,
    cmdclass=versioneer.get_cmdclass(),
    description=rekall_description,
    long_description=open("README.md").read(),
    license="GPL",
    url="https://www.rekall-forensic.com/",
    author="The Rekall team",
    author_email="rekall-discuss@googlegroups.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],

    # This requires an exact version to ensure that installing the meta package
    # pulls in tested dependencies.
    install_requires=[
        "rekall-core == %s" % MY_VERSION,
        "rekall-gui == %s" % MY_VERSION,
    ],
)



#!/usr/bin/python
"""This is a CGI handler for github push webhooks.

To install:

1. Copy this file to your server cgi-bin directory (e.g. /usr/lib/cgi-bin/).\
2. Modify the SECRET below.
3. create a new github web hook with the URL to this cgi file and update the
   SECRET.

All this file does is write a control file in the specified location when github
sends a push notification.
"""
import json
import hashlib
import hmac
import os
import sys

# Modify these for your installation.
SECRET = "iamasecret"
CONTROL = "/home/rekalltest/control"


data = sys.stdin.read()

print("""Content-type: text/html

<html><h1>
""")

# Verify the hmac.
hmac_sig = hmac.HMAC(SECRET, data, hashlib.sha1).hexdigest()
if hmac_sig != os.environ['HTTP_X_HUB_SIGNATURE'].split("=")[1]:
    print("Denied")
else:
    # Create the control file to kick off the test run.
    print("Scheduling run.")
    data = json.loads(data)
    data["action"] = "start"

    with open(CONTROL, "wb") as fd:
        fd.write(json.dumps(data))
